@@ -196,6 +196,8 @@ lollipop3d(vv$spacelag, vv$timelag, vv$gamma,  main="simplified sum-metric model
 ## kriging using the sum-metric model
 
 data(air)
+if (!exists("rural"))
+	rural = STFDF(stations, dates, data.frame(PM10 = as.vector(air)))
 rr <- rural[,"2005-06-01/2005-06-10"]
 rr <- as(rr, "STSDF")
 
@@ -218,4 +220,4 @@ fitSumMetricModel_utm$stAni <- fitSumMetricModel_utm$stAni * 1000
 DE_kriged <- krige(PM10~1, rr_utm, DE_pred, fitSumMetricModel_utm)
 stplot(DE_kriged, col.regions=bpy.colors(), 
        sp.layout = list("sp.polygons", DE_utm, col = grey(.5), first = FALSE, lwd=2),
-       main="spatio-temporal interpolated PM10 concentrations")
\ No newline at end of file
+       main="spatio-temporal interpolated PM10 concentrations")
